@@ -1,22 +1,21 @@
 from ._collection import (
     SpintopTestRecordCollection, 
-    SpintopFlatTestRecordBuilder,
-    SpintopSerializedFlatTestRecord,
+    SpintopTestRecordBuilder,
+    SpintopSerializedTestRecord,
     SpintopTestRecord,
     SpintopTestRecordView,
     SpintopSerializedTestRecordCollection
 )
 
 from ._base import (
-    serialized_get_test_uuid,
-    DefaultPrimitiveView,
     BaseDataClass,
     TestIDRecord,
-    TestRecordSummary, 
     FeatureRecord,
     MeasureFeatureRecord,
     PhaseFeatureRecord,
     DutIDRecord,
+    TestbenchIDRecord,
+    TestRecordSummary,
     DutOp,
     OutcomeData,
     NO_VERSION
