@@ -10,18 +10,22 @@ from datetime import datetime
 from dataclasses import dataclass, fields, _MISSING_TYPE, MISSING, field, asdict
 from typing import Union, List, Optional
 
+from ..serialization import cls_of_obj
+
 from ..base import (
-    cls_of_obj,
     BaseDataClass, 
-    register_type, 
     _fields_cache,
-    model_dataclass
+    model_property
 )
 
+from ..persistence import PersistenceIDRecord
+
 from ..view import ComplexPrimitiveView
 
+from spintop.utils import isnan, utcnow_aware
+
 NAME_SEPARATOR = ':'
-NO_VERSION = '0'
+NO_VERSION = 'none'
 
 def create_uuid():
     return uuid4()
@@ -34,7 +38,7 @@ def uuid_to_slug(uuidstring):
     
     return urlsafe_b64encode(uuid.bytes).rstrip(b'=').decode('ascii')
 
-def test_uuid_generator(testbench_name=None, start_datetime=None, dut_record=None):
+def test_uuid_generator(testbench_record=None, start_datetime=None, dut_record=None):
     datetime_format = None
 
     if start_datetime:
@@ -44,7 +48,7 @@ def test_uuid_generator(testbench_name=None, start_datetime=None, dut_record=Non
         # Not enough info for unique id. Generate one.
         value =  uuid_to_slug(create_uuid())
     else:
-        value =  f"{testbench_name}.{datetime_format}"
+        value =  f"{str(testbench_record)}.{datetime_format}"
 
     if dut_record.id:
         value = f"{value}.{dut_record.id}-{dut_record.version}"
@@ -55,12 +59,6 @@ def test_uuid_generator(testbench_name=None, start_datetime=None, dut_record=Non
 def slug_to_uuid(slug):
     return str(UUID(bytes=urlsafe_b64decode(slug + '==')))
 
-def serialized_get_test_uuid(record):
-    try:
-        return record['test_id']['test_uuid']
-    except KeyError:
-        raise ValueError("test_id.test_uuid not in record! {!r}".format(record))
-
 def compute_stats(features):
     name_tuple_to_count_map = defaultdict(int)
 
@@ -76,14 +74,14 @@ def compute_stats(features):
         feature.feature_count = name_tuple_to_count_map[feature.name_tuple]
 
 
-@model_dataclass()
-@register_type('dut')
-class DutIDRecord(BaseDataClass):
+class VersionIDRecord(BaseDataClass):
     id: Optional[str]
     version: Optional[str]
     
     @classmethod
     def create(cls, id_or_dict, version=None):
+        if isinstance(id_or_dict, cls):
+            return id_or_dict
         if isinstance(id_or_dict, Mapping):
             id = id_or_dict.get('id', None)
             version = id_or_dict.get('version', None)
@@ -103,51 +101,66 @@ class DutIDRecord(BaseDataClass):
         version_match = self.version == other.version if self.version is not None else True
         return id_match and version_match
 
-@model_dataclass()
-@register_type('test_id')
-class TestIDRecord(BaseDataClass):
-    testbench_name: str = 'default'
-    dut: DutIDRecord
-    test_uuid: str
-    tags: dict = dict
-    start_datetime: datetime = datetime.utcnow
+    def __str__(self):
+        if self.version and self.version != NO_VERSION:
+            return f'{self.id}-{self.version}'
+        elif self.id:
+            return str(self.id)
+        else:
+            return 'default'
 
-    def __post_init__(self):
-        if self.test_uuid is None:
-            self.test_uuid = test_uuid_generator(self.testbench_name, self.start_datetime, self.dut)
+class TestbenchIDRecord(VersionIDRecord):
+    pass
+
+class DutIDRecord(VersionIDRecord):
+    pass
+
+class OutcomeData(BaseDataClass):
+    message: str = ''
+    is_pass: bool = True
+    is_skip: bool = False
+    is_abort: bool = False
+    is_ignore: bool = False
+    
+    def __bool__(self):
+        return self.is_pass and not self.is_abort
 
     @classmethod
-    def create(cls, testbench_name, dut, start_datetime, tags=None):
-        dut_record = DutIDRecord.create(dut)
-        return cls(
-            testbench_name=testbench_name,
-            dut=dut_record,
-            test_uuid=None, # will be generated in post init
-            start_datetime=start_datetime,
-            tags= tags if tags else {}
-        )
-        
-    
-    def add_tag(self, key, value=True):
-        self.tags[key] = value
-        
-    def remove_tag(self, key):
-        del self.tags[key]
+    def create(cls, outcome):
+        if isinstance(outcome, cls):
+            pass # already correct type
+        elif isinstance(outcome, bool):
+            outcome = cls(is_pass=outcome)
+        elif isinstance(outcome, Mapping):
+            outcome = cls(**outcome)
+        else:
+            raise TypeError((
+    'Outcome (%s) should either be a boolean that indicates if this test passed or failed, ' 
+    'or a dictionnary of one or more of the following attributes: %s'
+            ) % (outcome, [f.name for f in cls.dataclass_fields()]))
+        return outcome
+
+    def impose_upon(self, other_outcome):
+        if not self.is_ignore:
+            # Pass propagates by falseness. False will propagate up, not True
+            other_outcome.is_pass = other_outcome.is_pass and self.is_pass
+
+            # Abort propagates by trueness. True will propagate up, not False
+            other_outcome.is_abort = other_outcome.is_abort or self.is_abort
+
+            # Abort propagates by trueness. True will propagate up, not False
+            other_outcome.is_skip = other_outcome.is_skip or self.is_skip
 
-@model_dataclass()
-@register_type('features_no_data')
 class FeatureRecordNoData(BaseDataClass):
-    """ Feature ID is determined by its name and the test_id's test_uuid.
+    """ Feature ID is determined by its name.
     """
     oid: Optional[str]
-    test_id: TestIDRecord
-    name: str
+    name: Optional[str] = ''
     description: str = ''
     version: int = 0
     depth: int = 0
     index: int = 0
-    feature_count: int = 0 # Number of children
-    ancestors: List[str]
+    ancestors: List[str] = list
     original: bool = True
     
     @classmethod
@@ -157,7 +170,7 @@ class FeatureRecordNoData(BaseDataClass):
         return super().defaults(**others)
 
     @classmethod
-    def get_data_fields(cls):
+    def data_dataclass_fields(cls):
         """ The data fields are all fields except the ones declare up to this cls
         in the cls MRO.
         
@@ -171,62 +184,79 @@ class FeatureRecordNoData(BaseDataClass):
         return tuple(self.ancestors) + (self.name,)
 
     @property
-    def complete_name(self):
+    def complete_name(self) -> str:
         return NAME_SEPARATOR.join(self.name_tuple)
-
-    def as_dict(self):
-        """ Transforms this object recursively into a dict. Adds the static
-        _type field to the data.."""
-        data = super(FeatureRecordNoData, self).as_dict()
-        return data
     
-@model_dataclass()
-@register_type('outcome')
-class OutcomeData(BaseDataClass):
-    is_pass: bool = True
-    is_skip: bool = False
-    is_abort: bool = False
-    is_ignore: bool = False
-    
-    def __bool__(self):
-        return self.is_pass and not self.is_abort
-
-    def impose_upon(self, other_outcome):
-        if not self.is_ignore:
-            # Pass propagates by falseness. False will propagate up, not True
-            other_outcome.is_pass = other_outcome.is_pass and self.is_pass
-
-            # Abort propagates by trueness. True will propagate up, not False
-            other_outcome.is_abort = other_outcome.is_abort or self.is_abort
-
-            # Abort propagates by trueness. True will propagate up, not False
-            other_outcome.is_skip = other_outcome.is_skip or self.is_skip
-
-@model_dataclass()
-@register_type('features')
 class FeatureRecord(FeatureRecordNoData):
-    user_data: dict # TODO Replace with data_view with descriptor protocol
-    outcome: OutcomeData
+    user_data: dict = dict
+    outcome: OutcomeData = OutcomeData
+    feature_count: int = 0 # Number of children
     
+    def __post_init__(self):
+        super().__post_init__()
+        self.outcome = OutcomeData.create(self.outcome)
+
+    @property
+    def total_feature_count(self):
+        return self.feature_count
+
     @classmethod
     def defaults(cls, **others):
         others['user_data'] = others.get('user_data', {})
         others['feature_count'] = others.get('feature_count', 0)
         return super(FeatureRecord, cls).defaults(**others)
     
-@model_dataclass()
-@register_type('phases')
 class PhaseFeatureRecord(FeatureRecord):
     duration: Optional[float] = None
     
-@model_dataclass()
-@register_type('measures')
+class TestIDRecord(PersistenceIDRecord, PhaseFeatureRecord):
+    testbench: TestbenchIDRecord = TestbenchIDRecord
+    station: TestbenchIDRecord = TestbenchIDRecord
+    dut: DutIDRecord = DutIDRecord
+    operator: str = ''
+    stage: str = 'prod'
+
+    def __post_init__(self):
+        super().__post_init__()
+        self.dut = DutIDRecord.create(self.dut)
+        self.testbench = TestbenchIDRecord.create(self.testbench)
+        self.station = TestbenchIDRecord.create(self.station)
+        
+        for key, value in self.tags.items():
+            self.add_tag(key, value)
+
+        if self.uuid is None:
+            self.uuid = test_uuid_generator(self.testbench, self.start_datetime, self.dut)
+
+    @property
+    def testbench_name(self) -> Optional[str]:
+        return self.testbench.id
+
+    @testbench_name.setter
+    def testbench_name(self, value):
+        self.testbench.id = value
+
+    def add_tag(self, key, value=True):
+        self.tags[key] = tag_value_sanitizer(value)
+        
+    def remove_tag(self, key):
+        del self.tags[key]
+
+TestRecordSummary = TestIDRecord # backward compat
+
+def tag_value_sanitizer(value):
+    if isinstance(value, float) and isnan(value):
+        # NaN to None.
+        value = None
+
+    return value
+
 class MeasureFeatureRecord(FeatureRecord):
     value_f: Optional[float]
     value_s: Optional[str]
 
     @property
-    def value(self):
+    def value(self) -> Union[float, str, None]:
         if self.value_f is not None:
             return self.value_f
         else:
@@ -243,25 +273,10 @@ class MeasureFeatureRecord(FeatureRecord):
         else:
             raise ValueError('Only string or numeric types are supported. Received {!r} of type {}'.format(value, type(value)))
     
-    
-@model_dataclass()
-@register_type('top_level')
-class TestRecordSummary(PhaseFeatureRecord):
-
-    @property
-    def total_feature_count(self):
-        return self.feature_count
-
-    @classmethod
-    def get_data_fields(cls):
-        base = super(TestRecordSummary, cls).get_data_fields()
-        return list(base) + [FeatureRecord.test_id]
-    
-@model_dataclass()
 class DutOp(BaseDataClass):
     dut_match: DutIDRecord
     dut_after: DutIDRecord
-    op_datetime: datetime = datetime.utcnow
+    op_datetime: datetime = utcnow_aware
     
     @classmethod
     def create(cls, dut_match, dut_after, op_datetime):
@@ -286,15 +301,4 @@ class DutOp(BaseDataClass):
         if new_dut.id is None:
             new_dut.id = dut.id
         return new_dut
-        
-        
-def default_cls_missing_view_fn(data_obj):
-    cls = cls_of_obj(data_obj)
-    return cls.get_default_view()
-        
-class DefaultPrimitiveView(ComplexPrimitiveView):
-    def __init__(self):
-        super(DefaultPrimitiveView, self).__init__(
-            BaseDataClass, 
-            cls_missing_view_fn=default_cls_missing_view_fn
-        )
\ No newline at end of file
+        
\ No newline at end of file
