@@ -1,22 +1,26 @@
 from collections.abc import Mapping
 from collections import OrderedDict
-from itertools import chain, starmap
 
 from .meta import MetaField
-    
+
+from spintop.utils.dict_ops import (
+    update, 
+    deepen_value_in_dict,
+    deepen_dict,
+    flatten_dict,
+    DEFAULT_PLACEHOLDER
+)
+from spintop.utils.get_recursive import GetRecursive
+
 class _MetaFieldAccessor(object):
     def __init__(self, meta_field):
         self.field = meta_field
 
     def __call__(self, obj):
-        return obj.getattr_from_field(self.field)
+        return self.field.get_value(obj)
 
-class _PythonAccessor(object):
-    def __init__(self, accessor):
-        self.accessor = accessor
-
-    def __call__(self, obj):
-        return self.accessor.__get__(obj, type(obj))
+    def __repr__(self):
+        return repr(self.field)
 
 class _ValueAccessor(object):
     def __init__(self, value):
@@ -24,7 +28,19 @@ class _ValueAccessor(object):
     
     def __call__(self, obj):
         return self.value
+
+    def __repr__(self):
+        return f'Value({repr(self.value)})'
     
+class _CallableAccessor(object):
+    def __init__(self, fn):
+        self.fn = fn
+
+    def __call__(self, obj):
+        return self.fn(GetRecursive(obj))
+
+    def __repr__(self):
+        return f'Call({self.fn.__name__}: {self.fn!r})'
     
 class DataClassPrimitiveView(object):
     def __init__(self, key_values_mapping={}):
@@ -36,29 +52,35 @@ class DataClassPrimitiveView(object):
 
         self._accessors_mapping = _build_accessors(key_values_mapping)
         
-    def apply(self, data_obj, flatten_dict=False, attribute_error_policy='ignore'):
+    def apply(self, data_obj, flatten_dict=False, key_prefix=None, attribute_error_policy='ignore'):
         result = {}
+        prefix_accessor = None
+
+        if key_prefix:
+            prefix_accessor = _build_single_accessor(key_prefix)
+
+        raise_attribute_errors = attribute_error_policy != 'ignore'
         for accessor_key, accessor_value in self._accessors_mapping.items():
-            try:
-                key = _apply_single_accessor(accessor_key, data_obj)
-                value = _apply_single_accessor(accessor_value, data_obj)
-            except AttributeError:
-                if attribute_error_policy == 'ignore':
-                    continue
-                else:
-                    raise
-            
-            if isinstance(key, tuple) and not flatten_dict:
-                # Push value deep into a nested dictionnary
-                # key = (x, y, z)
-                # result.update({x: {y: {z: value}}})
-                update(result, deepen_value_in_dict(key, value))
+            # Set to none if does not exist.
+            key = None
+
+            if prefix_accessor:
+                complete_key = prefix_accessor + accessor_key
             else:
-                # or keep as is (flat)
-                # key = (x, y, z)
-                # result[(x, y, z)] = value
-                result[key] = value
+                complete_key = accessor_key
+
+            try:
+                key = _apply_single_accessor(complete_key, data_obj)
+                value = _apply_single_accessor(accessor_value, data_obj)
+            except (AttributeError, KeyError) as e:
+                if raise_attribute_errors:
+                    raise
+                else:
+                    _apply_key_value(result, key, DEFAULT_PLACEHOLDER, flatten_dict)
+                    continue
             
+            _apply_key_value(result, key, value, flatten_dict)
+        
         return result
 
 def _build_accessors(key_values_mapping):
@@ -68,12 +90,12 @@ def _build_accessors(key_values_mapping):
 def _build_single_accessor(obj):
     if isinstance(obj, tuple):
         return tuple(_build_single_accessor(subobj) for subobj in obj)
-    elif isinstance(obj, MetaField):
+    elif isinstance(obj, MetaField) or isinstance(obj, property):
         return _MetaFieldAccessor(obj)
-    elif isinstance(obj, property):
-        return _PythonAccessor(obj)
+    # elif isinstance(obj, property):
+    #     return _PythonAccessor(obj)
     elif callable(obj):
-        return obj # Obj is an accessor
+        return _CallableAccessor(obj) # Obj is an accessor
     else:
         return _ValueAccessor(obj) # Will always return obj
     
@@ -83,9 +105,31 @@ def _apply_single_accessor(accessor, obj):
     else:
         return accessor(obj)
 
-def default_cls_missing_view_fn(cls):
+def default_cls_missing_view_fn(data_obj):
     raise NotImplementedError()
 
+def _apply_key_value(_dict, key, value, flatten_dict):
+    if isinstance(value, Mapping):
+        # Push keys from a value dictionnary into the key itself.
+        # the rest of the code will handle flatten_dict well enough
+        # I.e. if value = {'foo': 'bar', 'bar':'foo'}, this will call
+        # this function with 
+        #   key=foo, value=bar AND 
+        #   key=bar, value=foo
+        for subkey, subvalue in value.items():
+            _apply_key_value(_dict, key + (subkey,), subvalue, flatten_dict)
+
+    elif isinstance(key, tuple) and not flatten_dict:
+        # Push value deep into a nested dictionnary
+        # key = (x, y, z)
+        # result.update({x: {y: {z: value}}})
+        update(_dict, deepen_value_in_dict(key, value))
+    else:
+        # or keep as is (flat)
+        # key = (x, y, z)
+        # result[(x, y, z)] = value
+        _dict[key] = value
+
 class ComplexPrimitiveView(object):
     def __init__(self, base_cls, cls_missing_view_fn=None):
         if cls_missing_view_fn is None: cls_missing_view_fn = default_cls_missing_view_fn
@@ -111,42 +155,41 @@ class ComplexPrimitiveView(object):
                 
         return closest_parent
     
-    def _find_view(self, cls):
+    def _find_view(self, data_obj):
+        cls = data_obj.__class__
         best_match = self._find_closest_parent(cls)
         if best_match:
             return self.custom_views[best_match]
         else:
-            return self.cls_missing_view_fn(cls)
+            return self.cls_missing_view_fn(data_obj)
     
     def add_view(self, cls, view_mapping):
         self.custom_views[cls] = DataClassPrimitiveView(view_mapping)
         
-    def apply(self, root_data_obj, flatten_dict=False, key_prefix=None):
+    def apply(self, root_data_obj, flatten_dict=False, key_prefix=None, **apply_kwargs):
         flatten_key_prefix = None
         if flatten_dict:
             if key_prefix is None:
                 flatten_key_prefix = ()
             else:
                 flatten_key_prefix = key_prefix
-                
-        result = self._apply_data_obj(root_data_obj, flatten_key_prefix=flatten_key_prefix)
         
+        result = self._apply_data_obj(root_data_obj, flatten_key_prefix=flatten_key_prefix, **apply_kwargs)
+
         if not flatten_dict and key_prefix:
             return deepen_value_in_dict(key_prefix, result)
         else:
             return result
     
-        
-    
-    def _apply_data_obj(self, data_obj, flatten_key_prefix=None):
-        mapping = self._find_view(data_obj.__class__).apply(data_obj, flatten_dict=flatten_key_prefix is not None)
+    def _apply_data_obj(self, data_obj, flatten_key_prefix=None, **apply_kwargs):
+        mapping = self._find_view(data_obj).apply(data_obj, flatten_dict=flatten_key_prefix is not None, **apply_kwargs)
         
         flatten_key_prefix_built = _build_single_accessor(flatten_key_prefix)
         flatten_key_prefix_applied = _apply_single_accessor(flatten_key_prefix_built, data_obj)
-        
-        return self._apply_mapping(mapping, flatten_key_prefix=flatten_key_prefix_applied)
+
+        return self._apply_mapping(mapping, flatten_key_prefix=flatten_key_prefix_applied, **apply_kwargs)
     
-    def _apply_mapping(self, mapping, flatten_key_prefix=None):
+    def _apply_mapping(self, mapping, flatten_key_prefix=None, **apply_kwargs):
         result = OrderedDict()
         
         for key, value in mapping.items():
@@ -158,11 +201,11 @@ class ComplexPrimitiveView(object):
             discard_false_subvalue = False 
             
             if isinstance(value, self.base_cls):
-                subvalue = self._apply_data_obj(value, flatten_key_prefix=flat_key)
+                subvalue = self._apply_data_obj(value, flatten_key_prefix=flat_key, **apply_kwargs)
                 # If bool(subvalue) is False, then the sub-view returned an empty mapping or None. Do not consider it.
                 discard_false_subvalue = True 
             elif isinstance(value, Mapping):
-                subvalue = self._apply_mapping(value, flatten_key_prefix=flat_key)
+                subvalue = self._apply_mapping(value, flatten_key_prefix=flat_key, **apply_kwargs)
                 # If bool(subvalue) is False, the mapping is empty or none. Do not consider it.
                 discard_false_subvalue = True 
             elif flat_key is not None:
@@ -196,48 +239,3 @@ def join_key_prefix(key_prefix, key):
         
     return key_prefix + key
 
-def deepen_value_in_dict(key_prefix, value):
-    result = {}
-    next_dict = result
-    for key in key_prefix[:-1]:
-        next_dict[key] = {}
-        next_dict = next_dict[key]
-    next_dict[key_prefix[-1]] = value
-    
-    return result
-
-def flatten_dict(dictionary):
-    """Flatten a nested dictionary structure"""
-
-    tuplify = lambda key: key if isinstance(key, tuple) else (key,)
-
-    def unpack(parent_key, parent_value):
-        """Unpack one level of nesting in a dictionary"""
-        try:
-            items = parent_value.items()
-        except AttributeError:
-            # parent_value was not a dict, no need to flatten
-            yield (parent_key, parent_value)
-        else:
-            for key, value in items:
-                yield (parent_key + tuplify(key), value)
-
-
-    # Put each key into a tuple to initiate building a tuple of subkeys
-    dictionary = {tuplify(key): value for key, value in dictionary.items()}
-
-    while True:
-        # Keep unpacking the dictionary until all value's are not dictionary's
-        dictionary = dict(chain.from_iterable(starmap(unpack, dictionary.items())))
-        if not any(isinstance(value, dict) for value in dictionary.values()):
-            break
-
-    return dictionary
-
-def update(d, u):
-    for k, v in u.items():
-        if isinstance(v, Mapping):
-            d[k] = update(d.get(k, {}), v)
-        else:
-            d[k] = v
-    return d
\ No newline at end of file
