@@ -1,11 +1,12 @@
 import dateutil.parser
 
-from collections import defaultdict, Sequence
+from collections.abc import Sequence
+from collections import defaultdict
 
 from spintop.models import (
-    SpintopTestRecordCollection
+    SpintopTestRecordCollection,
+    DutOp
 )
-from spintop.models.internal import DutOp
 
 from .base import Transformer
 
